C     PROGRAM PSPLOT
C
C     Program to read the generic plot file STRUPLO.XYZ output by 
C     Reinhard X. Fischer's program STRUPL91 and to generate the 
C     PostScript file STRUPLO.PS containing the plot.
C
C     Written by: Donald L. Ward
C                 Department of Chemistry
C                 Michigan State University
C                 East Lansing, MI 48824-1322
C
C     Version:    O4 November 1993
C
C     modified by A. Le Bail     March 1996
C                 for PC machine : BACKSPACE was not running
C
C
C     ------------------------------------------------------------------
C
C     The STRUPLO.XYZ input data file has the following structure and 
C     definitions:    ----- ---- ---- 
C
C     record 1 - (F10.4)
C      col.  1-10  scale factor
C
C     record 2 - (40X,A40) 
C      col. 41-80  title
C
C     record 3 - (F10.4,F11.4)
C      col.  1-10  X origin shift (in cm.)
C           11-21  Y origin shift (in cm.)
C      NOTE: the plot view frame has X,Y dimensions (in cm.) twice the 
C            origin shift values.
C
C     The first three records contain information common to the entire
C     drawing.   Thereafter, blocks of records deal with individual
C     features of the drawing.   Each block consists of:
C
C     record 1 - (I2,I3)
C      col.  1- 2  code number (= 4)
C      col.  3- 5  pen number (= 1, 2, 3, 4, 5, 6, 7, or 8)
C
C     records 2-n - (I2,2F11.4)
C      col.  1- 2  code number (= 2 for move with pen down)
C                              (= 3 for move with pen up)
C      col.  3-13  X coordinate to move to
C      col. 14-24  Y coordinate to move to
C
C     The file just ends without any special terminator record.
C
C     ------------------------------------------------------------------
C
C     The STRUPLO.PS output plot file has the following structure and 
C     definitions:   ------ ---- ----
C
C     record 1 - (Adobe PS level)
C      %!PS-Adobe-1.0
C
C     record 2 - (creator of this file)
C      %%Creator: PSPLOT version 04-Nov-93 (by Donald L. Ward)
C
C     record 3 - (title of this file)
C      %%Title: Plot file from STRUPL91 (by Reinhard X. Fischer)
C
C     record 4 - (date/time this program was run)
C      %%CreationDate: dd-mmm-yy hh:mm:ss
C
C     record 5 -
C      %%Pages: (atend)
C
C     record 6 -
C      %%DocumentFonts: (atend)
C
C     record 7 - (define, in points, 8.5 by 11 view frame)
C      %%BoundingBox: 0 0 612 792
C
C     record 8 -
C      %%EndComments
C
C     record 9 -
C      %%EndProlog
C
C     record 10 - (start first page)
C      %%Page: 1 1
C
C     record 11 -
C      %%PageResources: (atend)
C
C     record 12 - (select font)
C      /Times-Roman findfont
C      15 scalefont
C      setfont
C
C     record 13 - (move coordinates origin to center of page)
C      nnnn.nn nnnn.nn translate
C
C     record 13a - (rotate image 90-degrees for landscape mode)
C      90 rotate                      (ONLY if transformed to landscape)
C
C     record 13b - (move coordinates origin to center of page)
C      nnnn.nn nnnn.nn translate      (ONLY if transformed to landscape)
C
C     record 14 - (set position for title)
C      nnnn.nn nnnn.nn moveto
C
C     record 15 - (print title)
C      (aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa) show
C
C     records 16 to n - (block of records for one plot feature)
C      %Start new feature
C      nnnn.nn setlinewidth             (ONLY if Black & White selected)
C      nnnn.nn nnnn.nn nnnn.nn setrgbcolor      (ONLY if Color selected)
C      newpath
C       nnnn.nn  nnnn.nn moveto
C       nnnn.nn  nnnn.nn lineto
C          "        "       "
C      stroke
C
C    (repeat "records 16 to n" for each feature in plot)
C
C     record last-8
C      statusdict begin /manualfeed false store end
C
C     record last-7
C      showpage 
C
C     record last-6
C      %%PageTrailer
C
C     record last-5
C      %%PageResources: font Times-Roman
C
C     record last-4
C      %%Trailer
C
C     record last-3
C      %%DocumentFonts: Times-Roman
C
C     record last-2
C      %%Pages: 1
C
C     record last-1
C      %%DocumentNeededResources: font Times-Roman
C
C     record last
C      %%EOF
C
C     ------------------------------------------------------------------
C
C      CHARACTER  CHAR*1, IDATE*9, ITIME*8, TITLE*40
      CHARACTER  CHAR*1, TITLE*40
      LOGICAL QEX
      INTEGER  ICODE, ICOLOR, ICRT, IKBD, INFILE, IPASS, IPEN, OUTFILE
      REAL     COLOR(3,8), NIBS(8)
      REAL     SCALE, XCORD, XORGSHFT, YCORD, YORGSHFT
      DATA NIBS  / 0.10, 0.15, 0.23, 0.34, 0.51, 0.76, 1.14, 1.71/
      DATA COLOR / 0.00, 0.00, 0.00,   ! BLACK
     *             0.25, 0.25, 0.25,   ! GREY
     *             1.00, 0.00, 0.00,   ! RED
     *             0.00, 1.00, 0.00,   ! GREEN
     *             0.00, 0.00, 1.00,   ! BLUE
     *             1.00, 0.00, 1.00,   ! MAGENTA
     *             1.00, 1.00, 0.00,   ! YELLOW
     *             1.00, 0.25, 0.00/   ! ORANGE
C
C     Open input and output plot files
C
      INFILE = 1
      OUTFILE = 2
      IKBD = 5
      ICRT = 6
      OPEN (UNIT=INFILE,  FILE='STRUPLO.XYZ', STATUS='OLD')
      INQUIRE(FILE='STRUPLO.PS',EXIST=QEX)
      IF(QEX.EQ..FALSE.)GO TO 6000
      OPEN(UNIT=OUTFILE,FILE='STRUPLO.PS',STATUS='OLD')
      CLOSE(UNIT=OUTFILE,STATUS='DELETE')
6000  OPEN(UNIT=OUTFILE,FILE='STRUPLO.PS',STATUS='NEW')
C     *      CARRIAGECONTROL='LIST')
C
C     Get time/date information, write welcome message
C
C      CALL DATE (IDATE)
C      CALL TIME (ITIME)
C      WRITE (ICRT, 1000) IDATE, ITIME
C
C     Querry whether plot is to be Black & White (default) or Color
C
      ICOLOR = 0
      WRITE (ICRT, 1001)
      READ (IKBD, 1002) CHAR
      IF (CHAR .EQ. 'Y' .OR. CHAR .EQ. 'y') ICOLOR = 1
C
C     Write PostScript comment records 
C
      WRITE (OUTFILE, 1003)
      WRITE (OUTFILE, 1004)
      WRITE (OUTFILE, 1005)
C      WRITE (OUTFILE, 1006) IDATE, ITIME
      WRITE (OUTFILE, 1007)
      WRITE (OUTFILE, 1008)
      WRITE (OUTFILE, 1009)
      WRITE (OUTFILE, 1010)
      WRITE (OUTFILE, 1011)
      WRITE (OUTFILE, 1012)
      WRITE (OUTFILE, 1013)
C
C     Process three input header records
C
      READ (INFILE, 1014) SCALE
      READ (INFILE, 1015) TITLE
      READ (INFILE, 1014) XORGSHFT, YORGSHFT
C
C     Translate origin to center the drawing on the sheet of paper, 
C     scale, and change to landscape mode (if necessary); plot title 
C
      CALL SCALEROT (SCALE, ICRT, INFILE, OUTFILE, TITLE)
C
C     Process block of records for one feature of plot
C     ------------------------------------------------
C
      READ (INFILE,*) ICODE, IPEN
10    IF (ICOLOR .EQ. 0) WRITE (OUTFILE, 1017) NIBS(IPEN)
      IF (ICOLOR .EQ. 1) WRITE (OUTFILE, 1018) (COLOR(I,IPEN),I=1,3)
      IPASS = 0
20    READ (INFILE,*, END=60) ICODE, XCORD, YCORD
      GO TO (30, 40, 50), ICODE - 1
      GO TO 999
C
C     Code number = 2, move with "pen down"
C
30    XCORD = SCALE * XCORD
      YCORD = SCALE * YCORD
      WRITE (OUTFILE, 1020) XCORD, YCORD 
      IPASS = 1
      GO TO 20
C
C     Code number = 3, move with "pen up"
C
40    IF (IPASS .EQ. 1) WRITE (OUTFILE, 1021)
      IPASS = 0
      XCORD = SCALE * XCORD
      YCORD = SCALE * YCORD
      WRITE (OUTFILE, 1022) XCORD, YCORD
      GO TO 20
C
C     Code number = 4, backspace input file, start new block
C
50    IPEN=(XCORD+0.5)
      IF (IPASS .EQ. 1) WRITE (OUTFILE, 1021)
      GO TO 10
C
C     End-of-file encountered on input plot file, finish output file
C
60    WRITE (OUTFILE, 1023)
      WRITE (OUTFILE, 1024)
      WRITE (OUTFILE, 1025)
      WRITE (OUTFILE, 1026)
      WRITE (OUTFILE, 1027)
      WRITE (OUTFILE, 1028)
      WRITE (OUTFILE, 1029)
      WRITE (OUTFILE, 1030)
      WRITE (OUTFILE, 1031)
      CLOSE (INFILE)
      CLOSE (OUTFILE)
      WRITE (ICRT, 1032)
      STOP
C
C     Invalid code number encountered on input plot file, stop
C
999   STOP ' Invalid code number encountered on input file'
C
C     ------------------------------------------------------------------
C
1000  FORMAT (/,' PSPLOT - (version 04-Nov-93)',/,
C     *        '  current date: ',A9,/,
C     *        '      and time: ',1X,A8,//,
     *        ' Program to read the STRUPLO.XYZ generic plot file',
     *        ' produced by STRUPL91',/,
     *        '  and to generate the PostScript plot file STRUPLO.PS',/)
1001  FORMAT (' The default is a Black & White drawing.',/,
     *        '$Do you, instead, want a COLOR drawing?  (Y/N):')
1002  FORMAT (A1)
1003  FORMAT ('%!PS-Adobe-1.0')
1004  FORMAT ('%%Creator: PSPLOT version 04-Nov-93 (by Donald L.',
     *        ' Ward)')
1005  FORMAT ('%%Title: Plot file from STRUPL91 (by Reinhard X.',
     *        ' Fischer)')
C1006  FORMAT ('%%CreationDate:',A9,1X,A8)
1007  FORMAT ('%%Pages: (atend)')
1008  FORMAT ('%%DocumentFonts: (atend)')
1009  FORMAT ('%%BoundingBox: 0 0 612 792')
1010  FORMAT ('%%EndComments')
1011  FORMAT ('%%EndProlog')
1012  FORMAT ('%%Page: 1 1')
1013  FORMAT ('%%PageResources: (atend)')
1014  FORMAT (F10.4,F11.4)
1015  FORMAT (40X,A40)
1016  FORMAT (I2,I3)
1017  FORMAT ('%Start new feature',/,
     *        F8.2,' setlinewidth')
1018  FORMAT ('%Start new feature',/,
     *        3F8.2,' setrgbcolor')
1019  FORMAT (I2,2F11.4)
1020  FORMAT (2F8.2,' lineto')
1021  FORMAT ('stroke')
1022  FORMAT ('newpath',/,2F8.2,' moveto')
1023  FORMAT ('statusdict begin /manualfeed false store end')
1024  FORMAT ('showpage')
1025  FORMAT ('%%PageTrailer')
1026  FORMAT ('%%PageResources: font Times-Roman')
1027  FORMAT ('%%Trailer')
1028  FORMAT ('%%DocumentFonts: Times-Roman')
1029  FORMAT ('%%Pages: 1')
1030  FORMAT ('%%DocumentNeededResources: font Times-Roman')
1031  FORMAT ('%%EOF')
1032  FORMAT (' PSPLOT Completed',/)
      END
      SUBROUTINE SCALEROT (SCALE, ICRT, INFILE, OUTFILE, TITLE)
C
C     Routine to scale plot coordinates as "points" (and rotate image by 
C     90 degrees, if necessary) to best place plot in 8.0 x 10.5 area on
C     an 8.5 x 11 sheet of paper.   Translate the origin by x=306, y=396
C     "points" to put plot origin in the middle of the sheet.
C
C     ------------------------------------------------------------------
C
      CHARACTER  TITLE*40
      INTEGER  ICODE, ICRT, INFILE, IPEN, IPORT, OUTFILE
      REAL     DX, DY, SCALE, SCALEX, SCALEY, TX, TY, XCORD, XMAX
      REAL     XMIN, YCORD, YMAX, YMIN
C
C     Examine entire plot file to determine +-X and +-Y extremes
C
      XMAX = -10000.
      XMIN =  10000.
      YMAX = -10000.
      YMIN =  10000.
10    READ (INFILE, *) ICODE, IPEN
20    READ (INFILE, *, END=50) ICODE, XCORD, YCORD
      GO TO (30, 30, 40), ICODE - 1
      STOP ' Invalid code number encountered on input file'
30    IF (XCORD .LT. XMIN) XMIN = XCORD
      IF (XCORD .GT. XMAX) XMAX = XCORD
      IF (YCORD .LT. YMIN) YMIN = YCORD
      IF (YCORD .GT. YMAX) YMAX = YCORD
40    GO TO 20
C
C     Reposition input file for use by main program
C
50    REWIND INFILE
      READ (INFILE, 1000) ICODE
      READ (INFILE, 1000) ICODE
      READ (INFILE, 1000) ICODE
C
C     Determine scale factor (assuming 8 X 10.5 area) and whether to
C     remain in portrait mode or rotate image into landscape mode.
C
      IF ((XMAX - XMIN) .LT. (YMAX - YMIN)) THEN 
        IPORT = 0  ! Will remain in portrait mode
        SCALEX = 576./(XMAX - XMIN)
        SCALEY = 756./(YMAX - YMIN)
        SCALE = SCALEX
        IF (SCALEY .LT. SCALE) SCALE = SCALEY
      ELSE
        IPORT = 1  ! Will rotate into landscape mode
        SCALEX = 756./(XMAX - XMIN)
        SCALEY = 576./(YMAX - YMIN)
        SCALE = SCALEX
        IF (SCALEY .LT. SCALE) SCALE = SCALEY
      ENDIF
C
C     Determine origin off-set from center of the 8.5 x 11 sheet of paper
C
      DX = (SCALE * (XMAX + XMIN) / 2.)
      DY = (SCALE * (YMAX + YMIN) / 2.)
C
C     Reduce scale by 10%
C
      SCALE = SCALE * 0.9
C
C     Translate origin, (rotate into landscape,) plot title at top of page
C
      WRITE (OUTFILE, 1002)
C     -- portrait mode
      IF (IPORT .EQ. 0) THEN
        WRITE (OUTFILE, 1003) 306. - DX, 396. - DY
        TX = -290. + DX
        TY =  369. + DY
        WRITE (OUTFILE, 1004) TX, TY 
      ENDIF
C     -- landscape mode
      IF (IPORT .EQ. 1) THEN
        WRITE (OUTFILE, 1003) 306., 396.
        WRITE (OUTFILE, 1005) ! rotate image by 90 degrees 
        WRITE (OUTFILE, 1003) - DX, - DY
        TX = -380. + DX
        TY =  279. + DY
        WRITE (OUTFILE, 1004) TX, TY
      ENDIF
C
c *** NEED TO CHECK (and correct) whether the 40 characters of title contain
c *** UNBALANCED parentheses (this will crash the PostScript printing!!)
c
      CALL CHKTITLE (TITLE)
      WRITE (OUTFILE, 1006) TITLE
      RETURN
C
1000  FORMAT (I2,I3)
1001  FORMAT (I2,2F11.4)
1002  FORMAT ('/Times-Roman findfont',/,
     *        '15 scalefont',/,
     *        'setfont')
1003  FORMAT (2F7.2,' translate')
1004  FORMAT (2F7.2,' moveto')
1005  FORMAT ('90 rotate')
1006  FORMAT ('(',A40,') show')
      END
      SUBROUTINE CHKTITLE (TITLE)
C
C     CHECK (and correct) the 40 characters of title so that it does NOT
C     contain UNBALANCED parentheses (otherwise, the PostScript printing
C     will crash!!)
C
      CHARACTER  ATITLE(40)*1, BTITLE*40, IREPL*1, ISYMB*1, TITLE*40
      INTEGER*4  ICL, ICR, IDIF
      EQUIVALENCE (ATITLE(1), BTITLE)
C
      BTITLE = TITLE
C
C     count numbers of left and right parentheses
C
      DO 10 I=1,40
        IF (ATITLE(I) .EQ. '(') ICL = ICL + 1
        IF (ATITLE(I) .EQ. ')') ICR = ICR + 1
10    CONTINUE
C
C     if parentheses are balanced, all's well, return
C
      IF (ICL .EQ. ICR) RETURN
C
C     replace offending parenthesis(es) with bracket(s)
C
      ISYMB = ')'
      IREPL = '}'
      IDIF = ICL - ICR
      IF (IDIF .GE. 1) ISYMB = '('
      IF (IDIF .GE. 1) IREPL = '{'
      IDIF = IABS(IDIF)
      DO 20 I=40,1,-1
        IF (ATITLE(I) .NE. ISYMB) GO TO 20
        ATITLE(I) = IREPL
        IDIF = IDIF - 1
        IF (IDIF .LE. 0) GO TO 30
20    CONTINUE
      STOP ' TROUBLE removing unbalanced parentheses from TITLE'
C
C     return
C
30    TITLE = BTITLE
      RETURN
C
      END
